using System;
using System.Collections;
using System.Windows.Forms;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Runtime.InteropServices;
using System.Diagnostics;
using ClientControls;

using Phoenix;

namespace SpringClient
{
	/// <summary>
	/// Summary description for BattleClient.
	/// </summary>
	public class BattleClient
	{
		[DllImport("User32.dll")]
		static extern Int32 SendMessage(IntPtr hWnd, Int32 wMsg, Int32 wParam, Int32 lParam);

		[DllImport("User32.dll")]
		static extern Int32 GetScrollPos(IntPtr hWnd, Int32 nBar);

		[DllImport("User32.dll")]
		static extern Boolean GetScrollRange(IntPtr hWnd, Int32 nBar, ref Int32 lpMinPos,
			ref Int32 lpMaxPos);

		private ConnectionManager		m_ConnectionManager;

		private ClientControls.ListViewEx m_lv;
		private RichTextBox		m_PrevMessages;
		private Timer			m_Timer;
		private bool			m_Stopped;
		private int			m_UnsyncedQueue = 0;

		private const int		MAXCONNECTIONS = 32;

		private BattleStateManager.BattleMemberInfo m_Self = null;

		public BattleClient(ClientControls.ListViewEx lv, RichTextBox prevMessages)
		{
			//
			// TODO: Add constructor logic here
			//
			m_Timer = new Timer();
			m_Timer.Interval = 4000;
			m_Timer.Tick += new EventHandler(m_Timer_Tick);
			m_Timer.Stop();
			m_lv = lv;
			m_PrevMessages = prevMessages;
			m_Stopped = false;
		}

		public bool Start(ConnectionManager cm)
		{
			try
			{

				if ( !cm.Connected )
					return false;
				m_ConnectionManager = cm;

				// here is where we send the unit syncing information

			
				byte temp = 0;
				while ( !UnitSync.UnitsProcessed() )
				{
					if ( temp > 5 )
					{
						MessageBox.Show("Waiting on UnitSync initializer");
					}
					System.Threading.Thread.Sleep(TimeSpan.FromSeconds(1.0));
				}

				string list = UnitSync.GetCurrentList();
				byte[] buffer = new byte[list.Length+3];
				buffer[0] = (byte)BATTLEPACK.PACK_SYNCLIST;
				BitConverter.GetBytes((ushort)list.Length).CopyTo(buffer, 1);
				//BitConverter.ToNetworkBytes((ushort)list.Length,buffer,1);
				Encoding.ASCII.GetBytes(list, 0, list.Length, buffer, 3);
				cm.Send(buffer);
				m_UnsyncedQueue++;

				m_Timer.Start();
				//		MessageBox.Show("Starting client");
				m_ConnectionManager.socketHandler.Read(1, new SocketHandler.delRead(ConnListener), Global.PrimaryForm);
			}
			catch ( Exception ex )
			{
				MessageBox.Show(ex.ToString());
			}
			return true;
		}

		public bool isConnected()
		{
			if ( m_ConnectionManager == null || !m_ConnectionManager.Connected )
				return false;
			return true;
		}

		public void Stop()
		{
			if ( m_ConnectionManager != null && m_ConnectionManager.Connected )
			{
				m_Stopped = true;
				lock ( m_ConnectionManager )
				{
					m_ConnectionManager.Send((byte)BATTLEPACK.PACK_LEAVE);
					m_ConnectionManager.Disconnect();
				}
			}
			UnitSync.Free();
		}

		private void ConnListener(byte[] buffer)
		{
			try
			{
				Debug.WriteLine("Received " + ((BATTLEPACK)buffer[0]).ToString());
				switch ( (BATTLEPACK)buffer[0] )
				{
					case BATTLEPACK.PACK_MESSAGE:
						m_ConnectionManager.socketHandler.Read(2, new SocketHandler.delRead(HandlePackBattleMessage), Global.PrimaryForm);
						break;
					case BATTLEPACK.PACK_LIST:
						HandlePackBattleList();
						break;
					case BATTLEPACK.PACK_CHANGETEAM:
						m_ConnectionManager.socketHandler.Read(5, new SocketHandler.delRead(HandlePackChangeTeam), Global.PrimaryForm);
						break;
					case BATTLEPACK.PACK_CHANGEALLYTEAM:
						m_ConnectionManager.socketHandler.Read(2, new SocketHandler.delRead(HandlePackChangeAllyTeam), Global.PrimaryForm);
						break;
					case BATTLEPACK.PACK_CHANGERESOURCES:
						m_ConnectionManager.socketHandler.Read(8, new SocketHandler.delRead(HandlePackResources), Global.PrimaryForm);
						break;
					case BATTLEPACK.PACK_CHANGEMAXUNITS:
						m_ConnectionManager.socketHandler.Read(4, new SocketHandler.delRead(HandlePackMaxUnits), Global.PrimaryForm);
						break;
					case BATTLEPACK.PACK_CHANGESTARTPOSTYPE:
						m_ConnectionManager.socketHandler.Read(4, new SocketHandler.delRead(HandlePackStartPosType), Global.PrimaryForm);
						break;
					case BATTLEPACK.PACK_CHANGEGAMEGOALTYPE:
						m_ConnectionManager.socketHandler.Read(4, new SocketHandler.delRead(HandlePackGameGoalType), Global.PrimaryForm);
						break;
					case BATTLEPACK.PACK_LEAVE:
						m_ConnectionManager.socketHandler.Read(4, new SocketHandler.delRead(HandlePackLeave), Global.PrimaryForm);
						break;
					case BATTLEPACK.PACK_NEWUSER:
						m_ConnectionManager.socketHandler.Read(7+CONSTANTS.MAXNAME+CONSTANTS.MAXCLAN, new SocketHandler.delRead(HandlePackNewUser), Global.PrimaryForm);
						break;
					case BATTLEPACK.PACK_READY:
		//				MessageBox.Show("RECEIVED READY");
						m_ConnectionManager.socketHandler.Read(5, new SocketHandler.delRead(HandlePackReady), Global.PrimaryForm);
						break;
					case BATTLEPACK.PACK_MAP:
						m_ConnectionManager.socketHandler.Read(CONSTANTS.MAXMAPNAME, new SocketHandler.delRead(HandlePackMap), Global.PrimaryForm);
						break;
					case BATTLEPACK.PACK_KEEPALIVE:
						m_ConnectionManager.socketHandler.Read(1, new SocketHandler.delRead(ConnListener), Global.PrimaryForm);
						break;
					case BATTLEPACK.PACK_START:
						HandlePackStart();
						break;
					case BATTLEPACK.PACK_CHANGESIDE:
						m_ConnectionManager.socketHandler.Read(2, new SocketHandler.delRead(HandlePackChangeSide), Global.PrimaryForm);
						break;
					case BATTLEPACK.PACK_ALLIANCE:
						m_ConnectionManager.socketHandler.Read(2, new SocketHandler.delRead(HandlePackAddAlliance), Global.PrimaryForm);
						break;
					case BATTLEPACK.PACK_REMOVEALLIANCE:
						m_ConnectionManager.socketHandler.Read(2, new SocketHandler.delRead(HandlePackRemoveAlliance), Global.PrimaryForm);
						break;
					case BATTLEPACK.PACK_BOOTED:
						Stop();
						m_Timer.Stop();
						m_Timer = null;
						Global.battleRoom.Close();
						MessageBox.Show("Booted from battle room");
						break;
					case BATTLEPACK.PACK_SYNCLIST:
						m_ConnectionManager.socketHandler.Read(2, new SocketHandler.delRead(HandlePackSyncList), Global.PrimaryForm);
						break;
					case BATTLEPACK.PACK_SYNCED:
						m_ConnectionManager.socketHandler.Read(4, new SocketHandler.delRead(HandlePackSynced), Global.PrimaryForm);
						break;
					default:
						throw new Exception(string.Format("Bad message {0} to battle room client", buffer[0]));
				}
			}
			catch ( SocketException )
			{
				MessageBox.Show("Connection lost - Battle Client");
				Global.battleRoom.Close();
				// ignore socket exceptions
				// typically means the connection was lost
			}
			catch ( Exception ex )
			{
				MessageBox.Show(ex.ToString());
				Global.battleRoom.Close();
			}
		}

		private void HandlePackSyncList(byte[] buffer)
		{
			try
			{
				ushort len = BitConverter.ToUInt16(buffer, 0);
				m_ConnectionManager.socketHandler.Read(len, new SocketHandler.delRead(HandlePackSyncListHelper), Global.PrimaryForm);
			}
			catch ( ArgumentOutOfRangeException are )
			{
				// this technically should never happen, but its the only
				// possible exception we are worried about
				MessageBox.Show(are.ToString());
			}
			catch ( NullReferenceException nre )
			{
				// same here
				MessageBox.Show(nre.ToString());
			}
		}

		private void HandlePackSyncListHelper(byte[] buffer)
		{
			try
			{
				string list = Encoding.ASCII.GetString(buffer);
				UnitSync.InstallClientDiff(list);
				m_UnsyncedQueue--;
			//	PostMessage(-1, string.Format("Unsynced counter: {0}", m_UnsyncedQueue));
				if ( m_UnsyncedQueue <= 0 )
				{
					m_ConnectionManager.Send((byte)BATTLEPACK.PACK_SYNCED);
					Global.battleRoom.m_BattleStateManager.SetSync(m_Self.m_ConnID);
					Global.battleRoom.PopulateDisabledUnits();
				}
				m_ConnectionManager.socketHandler.Read(1, new SocketHandler.delRead(ConnListener), Global.PrimaryForm);
			}
			catch ( ArgumentOutOfRangeException are )
			{
				// this technically should never happen, but its the only
				// possible exception we are worried about
				MessageBox.Show(are.ToString());
			}
			catch ( NullReferenceException nre )
			{
				// same here
				MessageBox.Show(nre.ToString());
			}
		}

		private void HandlePackSynced(byte[] buffer)
		{
			try
			{
				int id = BitConverter.ToInt32(buffer, 0);
				Global.battleRoom.m_BattleStateManager.SetSync(id);
				m_ConnectionManager.socketHandler.Read(1, new SocketHandler.delRead(ConnListener), Global.PrimaryForm);
			}
			catch ( ArgumentOutOfRangeException are )
			{
				// this technically should never happen, but its the only
				// possible exception we are worried about
				MessageBox.Show(are.ToString());
			}
			catch ( NullReferenceException nre )
			{
				// same here
				MessageBox.Show(nre.ToString());
			}
		}

		private void HandlePackNewUser(byte[] buffer)
		{
			try
			{
				int index = 0;
				int id = BitConverter.ToInt32(buffer, index);
				index += 4;
				string clan = Encoding.ASCII.GetString(buffer, index, CONSTANTS.MAXCLAN).TrimEnd("\0".ToCharArray());
				index += CONSTANTS.MAXCLAN;
				string name = Encoding.ASCII.GetString(buffer, index, CONSTANTS.MAXNAME).TrimEnd("\0".ToCharArray());
				index += CONSTANTS.MAXNAME;
				byte teamid = buffer[index++];
				byte internalid = buffer[index++];
				byte spec = buffer[index];
				bool spectator = false;
				if ( spec != 0 )
					spectator = true;
				Global.battleRoom.AddMember(spectator, id, false, (BattleStateManager.TeamColor)teamid, (int)internalid, name, clan, 0);
				Global.battleRoom.m_BattleStateManager.SetUnsyncAll();
		//		PostMessage(-1, string.Format("Unsynced counter: {0}", m_UnsyncedQueue));
				//m_UnsyncedQueue++;
				m_ConnectionManager.socketHandler.Read(1, new SocketHandler.delRead(ConnListener), Global.PrimaryForm);
			}
			catch ( Exception ex )
			{
				MessageBox.Show(ex.ToString());
			}
		}

		private void HandlePackStart()
		{
			try
			{
				Global.battleRoom.StartBattle();
				/*
				m_Timer.Stop();
				m_Timer = null;
				m_ConnectionManager.Disconnect();
				m_ConnectionManager = null;
				this.m_Stopped = true;
				Global.battleRoom.Close();
				*/
			//	this.SendReadyState(false);
				
			//	BattleStateManager.BattleMemberInfo m_Self = (BattleStateManager.BattleMemberInfo)Global.battleRoom.m_BattleStateManager.Members[0];
				if ( !m_Self.m_bSpectator )
					m_Self.m_checkReady.Checked = false;
				m_ConnectionManager.socketHandler.Read(1, new SocketHandler.delRead(ConnListener), Global.PrimaryForm);
			}
			catch ( Exception ex )
			{
				MessageBox.Show(ex.ToString());
			}
		}

		private void HandlePackMap(byte[] map)
		{
			try
			{
				string mapname = Encoding.ASCII.GetString(map).TrimEnd("\0".ToCharArray());
				//Global.battleRoom.SetMap(string.Format("{0}\\maps\\{1}.smf", Global.SpringDirectory, mapname), mapname);
				Global.battleRoom.SetMap(mapname + ".smf", mapname);
				m_ConnectionManager.socketHandler.Read(1, new SocketHandler.delRead(ConnListener), Global.PrimaryForm);
			}
			catch ( Exception ex )
			{
				MessageBox.Show(ex.ToString());
			}
		}

		private void HandlePackReady(byte[] buffer )
		{
			try
			{
				int id = BitConverter.ToInt32(buffer, 0);
				bool state = false;
				if ( buffer[4] != 0 )
					state = true;
				Global.battleRoom.ChangeReady(false, id, state);
				m_ConnectionManager.socketHandler.Read(1, new SocketHandler.delRead(ConnListener), Global.PrimaryForm);
			}
			catch ( Exception ex )
			{
				MessageBox.Show(ex.ToString());
			}
		}

		private void HandlePackLeave(byte[] buffer )
		{
			try
			{
				int id = BitConverter.ToInt32(buffer, 0);
				Global.battleRoom.m_BattleStateManager.RemoveMember(id);
				Global.battleRoom.m_BattleStateManager.SetUnsyncAll();
			//	PostMessage(-1, string.Format("Unsynced counter: {0}", m_UnsyncedQueue));
				//m_UnsyncedQueue++;
				m_ConnectionManager.socketHandler.Read(1, new SocketHandler.delRead(ConnListener), Global.PrimaryForm);
			}
			catch ( Exception ex )
			{
				MessageBox.Show(ex.ToString());
			}
		}

		public bool PostMessage( int userid, string data )
		{
				if ( Global.battleRoom.m_BattleStateManager.Members.ContainsKey(userid)  )
				{
					BattleStateManager.BattleMemberInfo ui = (BattleStateManager.BattleMemberInfo)
						Global.battleRoom.m_BattleStateManager.Members[userid];
					m_PrevMessages.AppendText(ui.m_Name.ToString() + ": ");
				}
				else
					m_PrevMessages.AppendText(userid.ToString() + ": ");

				m_PrevMessages.AppendText(data + "\n");
				m_PrevMessages.Update();

				if ( !m_PrevMessages.Focused )
					SendMessage(m_PrevMessages.Handle, Win32.WM_VSCROLL, Win32.SB_BOTTOM, 0);
			return true;
		}

		private void HandlePackChangeTeam(byte[] buffer)
		{
			try
			{
				int ID = BitConverter.ToInt32(buffer, 0);
				Global.battleRoom.ChangeTeam(ID, (BattleStateManager.TeamColor)buffer[4]);
				m_ConnectionManager.socketHandler.Read(1, new SocketHandler.delRead(ConnListener), Global.PrimaryForm);
			}
			catch ( Exception ex )
			{
				MessageBox.Show(ex.ToString());
			}
		}

		private void HandlePackChangeAllyTeam(byte[] buffer)
		{
			try
			{
				Debug.WriteLine(string.Format("ChangeAllyTeam Received - {0} to {1}", buffer[0].ToString(), buffer[1].ToString()));
				Global.battleRoom.ChangeAllyTeam(false, buffer[0], buffer[1]);
				m_ConnectionManager.socketHandler.Read(1, new SocketHandler.delRead(ConnListener), Global.PrimaryForm);
			}
			catch ( Exception ex )
			{
				MessageBox.Show(ex.ToString());
			}
		}

		private void HandlePackResources(byte[] buffer)
		{
			try
			{
				Debug.WriteLine(string.Format("ChangeResources Received - {0} to {1}", buffer[0].ToString(), buffer[1].ToString()));
				int metal=BitConverter.ToInt32(buffer, 0);
				int energy=BitConverter.ToInt32(buffer, 4);
				Global.battleRoom.ChangeResources(false, metal, energy);
				m_ConnectionManager.socketHandler.Read(1, new SocketHandler.delRead(ConnListener), Global.PrimaryForm);
			}
			catch ( Exception ex )
			{
				MessageBox.Show(ex.ToString());
			}
		}

		private void HandlePackMaxUnits(byte[] buffer)
		{
			try
			{
				Debug.WriteLine(string.Format("ChangeMaxUnits Received - {0} to {1}", buffer[0].ToString(), buffer[1].ToString()));
				int maxunits=BitConverter.ToInt32(buffer, 0);
				Global.battleRoom.ChangeMaxUnits(false, maxunits);
				m_ConnectionManager.socketHandler.Read(1, new SocketHandler.delRead(ConnListener), Global.PrimaryForm);
			}
			catch ( Exception ex )
			{
				MessageBox.Show(ex.ToString());
			}
		}

		private void HandlePackStartPosType(byte[] buffer)
		{
			try
			{
				Debug.WriteLine(string.Format("ChangeStartPosType Received - {0} to {1}", buffer[0].ToString(), buffer[1].ToString()));
				int type=BitConverter.ToInt32(buffer, 0);
				Global.battleRoom.ChangeStartPosType(false, type);
				m_ConnectionManager.socketHandler.Read(1, new SocketHandler.delRead(ConnListener), Global.PrimaryForm);
			}
			catch ( Exception ex )
			{
				MessageBox.Show(ex.ToString());
			}
		}

		private void HandlePackGameGoalType(byte[] buffer)
		{
			try
			{
				Debug.WriteLine(string.Format("ChangeGameGoalType Received - {0} to {1}", buffer[0].ToString(), buffer[1].ToString()));
				int type=BitConverter.ToInt32(buffer, 0);
				Global.battleRoom.ChangeGoalType(false, type);
				m_ConnectionManager.socketHandler.Read(1, new SocketHandler.delRead(ConnListener), Global.PrimaryForm);
			}
			catch ( Exception ex )
			{
				MessageBox.Show(ex.ToString());
			}
		}

		// when getting the battle list, we will allow for blocking I/O, since its SO much of a hassle otherwise
		private void HandlePackBattleList()
		{
			try
			{
//				Debug.WriteLine("Enter Pack Battle List");
					BattleStateManager bsm = Global.battleRoom.m_BattleStateManager;
					Global.battleRoom.m_BattleStateManager.Clear(false);
					byte[] count = new byte[1];

					m_ConnectionManager.Read(count);
					byte initialize = count[0];
				// get alliance list
				{
					m_ConnectionManager.Read(count);

					// numAllyTeams will always be MAXTEAMS.
					byte numAllyTeams = count[0];
					for ( byte i = 0; i < numAllyTeams; i++ )
					{
						m_ConnectionManager.Read(count);
						byte numAlliedTo = count[0];
						BattleStateManager.AllyTeamInfo ati = bsm.AllyTeams[i];
						ati.m_AlliedTo.Clear();
						ati.m_Members.Clear();
						for ( byte j = 0; j < numAlliedTo; j++ )
						{
							m_ConnectionManager.Read(count);
							ati.m_AlliedTo.Add(count[0]);
							//bsm.AddAlliance(i, count[0]);
						}
					}
				}
				
				// get team list
				{
					m_ConnectionManager.Read(count);
					// numTeams will always be MAXTEAMS
					byte numTeams = count[0];
					byte[] message = new byte[3];
					for ( byte i = 0; i < numTeams; i++ )
					{
						m_ConnectionManager.Read(message);
						// inserts a team into the appropriate position in the internal queue
						if ( message[0] != 0xFF && message[1] > bsm.AllyTeams[message[0]].m_Members.Count )
						{
							for ( byte j = (byte)bsm.AllyTeams[message[0]].m_Members.Count; j < message[1]; j++ )
							{
								bsm.AllyTeams[message[0]].m_Members.Insert(j, null);
							}
						}
						if ( message[0] != 0xFF )
							bsm.AllyTeams[message[0]].m_Members.Insert(message[1], bsm.Teams[i]);
						// and makes sure its internal id is set properly
						bsm.Teams[i].m_InternalID = message[1];
						bsm.Teams[i].m_AllyTeamID = message[0];
						bsm.Teams[i].m_Side = (BattleStateManager.SIDE)message[2];
					}
				}
				
				// get member list
				{
					m_ConnectionManager.Read(count);
					byte numMembers = count[0];
					byte[] message = new byte[CONSTANTS.MAXCLAN+CONSTANTS.MAXNAME+8];
					for ( byte i = 0; i < numMembers; i++ )
					{
						m_ConnectionManager.Read(message);
//						BattleStateManager.BattleMemberInfo bmi = new SpringClient.BattleStateManager.BattleMemberInfo();
						int id = BitConverter.ToInt32(message, 0);
						string clan = Encoding.ASCII.GetString(message, 4, CONSTANTS.MAXCLAN).TrimEnd("\0".ToCharArray());
						string name = Encoding.ASCII.GetString(message, 4+CONSTANTS.MAXCLAN, CONSTANTS.MAXNAME).TrimEnd("\0".ToCharArray());
						BattleStateManager.TeamColor team = (BattleStateManager.TeamColor)message[4+CONSTANTS.MAXCLAN+CONSTANTS.MAXNAME];
						// bool spectator = false;
						byte internalId = message[5+CONSTANTS.MAXCLAN+CONSTANTS.MAXNAME];
						byte ready = message[6+CONSTANTS.MAXCLAN+CONSTANTS.MAXNAME];
						byte spec = message[7+CONSTANTS.MAXCLAN+CONSTANTS.MAXNAME];
						bool spectator = false;
						if ( spec != 0 )
							spectator = true;

						if(initialize!=0)
						{
							if ( name == UserInfo.Name )
								m_Self = Global.battleRoom.AddMember(spectator, id, true, team, (int)internalId, name, clan, ready);
							else
								m_Self = Global.battleRoom.AddMember(spectator, id, false, team, (int)internalId, name, clan, ready);
						}
					}
				}
/*
				if (initialize!=0 && !m_Self.m_bSpectator )
				{
					foreach ( BattleStateManager.TeamInfo ti in Global.battleRoom.m_BattleStateManager.Teams )
					{
						if ( ti.m_Members.Count == 0 )
						{
							m_Self.m_CBTeam.SelectedIndex = ti.m_TeamID;
							break;
						}
					}
				}*/
				m_ConnectionManager.socketHandler.Read(1, new SocketHandler.delRead(ConnListener), Global.PrimaryForm);
//				Debug.WriteLine("Done battle list");
			}
			catch ( Exception ex )
			{
				MessageBox.Show(ex.ToString());
			}
		}

		public void SendChangeTeamMessage(BattleStateManager.TeamColor to)
		{
				lock ( m_ConnectionManager )
				{
					m_ConnectionManager.Send((byte)BATTLEPACK.PACK_CHANGETEAM);
					m_ConnectionManager.Send((byte)to);
				}
		}

		public void SendAlliance(byte to)
		{
			lock ( m_ConnectionManager )
			{
				m_ConnectionManager.Send((byte)BATTLEPACK.PACK_ALLIANCE);
				m_ConnectionManager.Send(to);
			}
		}

		public void SendRemoveAlliance(byte to)
		{
			lock ( m_ConnectionManager )
			{
				m_ConnectionManager.Send((byte)BATTLEPACK.PACK_REMOVEALLIANCE);
				m_ConnectionManager.Send(to);
			}
		}

		public void SendChangeSide(byte to)
		{
			lock ( m_ConnectionManager )
			{
				m_ConnectionManager.Send((byte)BATTLEPACK.PACK_CHANGESIDE);
				m_ConnectionManager.Send(to);
			}
		}

		public void SendChangeAllyTeamMessage(byte to)
		{
				lock ( m_ConnectionManager )
				{
					m_ConnectionManager.Send((byte)BATTLEPACK.PACK_CHANGEALLYTEAM);
					m_ConnectionManager.Send((byte)to);
				}
		}

		public void SendReadyState( bool state )
		{
				lock ( m_ConnectionManager )
				{
					m_ConnectionManager.Send((byte)BATTLEPACK.PACK_READY);
					if ( state )
						m_ConnectionManager.Send((byte)1);
					else
						m_ConnectionManager.Send((byte)0);
				}
		}

		public bool SendMessage( int userid, string data )
		{
			try
			{
				if ( data.Length >= 65536 )
					return false; // too long a message
				ushort len = (ushort)data.Length;
				byte[] msg = Encoding.ASCII.GetBytes(data);
				lock ( m_ConnectionManager )
				{
					m_ConnectionManager.Send((byte)BATTLEPACK.PACK_MESSAGE);
					m_ConnectionManager.Send(BitConverter.GetBytes(len));
					m_ConnectionManager.Send(BitConverter.GetBytes(userid));
					m_ConnectionManager.Send(msg);
				}
			}
			catch ( Exception ex )
			{
				MessageBox.Show(ex.ToString());
				return false;
			}

			return true;
		}

		public void HandlePackChangeSide(byte[] buffer)
		{
			try
			{
				byte team = buffer[0];
				byte newside = buffer[1];
				Global.battleRoom.Changeside(team, newside);
				m_ConnectionManager.socketHandler.Read(1, new SocketHandler.delRead(ConnListener), Global.PrimaryForm);
			}
			catch ( Exception ex )
			{
				MessageBox.Show(ex.ToString());
			}
		}

		public void HandlePackAddAlliance(byte[] buffer)
		{
			try
			{
				byte team = buffer[0];
				byte to = buffer[1];
				Global.battleRoom.AddAlliance(team, to);
				m_ConnectionManager.socketHandler.Read(1, new SocketHandler.delRead(ConnListener), Global.PrimaryForm);
			}
			catch ( Exception ex )
			{
				MessageBox.Show(ex.ToString());
			}
		}

		public void HandlePackRemoveAlliance(byte[] buffer)
		{
			try
			{
				byte team = buffer[0];
				byte to = buffer[1];
				Global.battleRoom.RemoveAlliance(team, to);
				m_ConnectionManager.socketHandler.Read(1, new SocketHandler.delRead(ConnListener), Global.PrimaryForm);
			}
			catch ( Exception ex )
			{
				MessageBox.Show(ex.ToString());
			}
		}

		public void HandlePackBattleMessage(byte[] buffer)
		{
			try
			{
				ushort len = BitConverter.ToUInt16(buffer, 0);
				int messagesize = len+4;
				// now read in the quote
				m_ConnectionManager.socketHandler.Read(messagesize, new SocketHandler.delRead(HandlePackBattleMessageQuote), Global.PrimaryForm);
			}
			catch ( Exception ex )
			{
				MessageBox.Show(ex.ToString());
			}
		}

		private void HandlePackBattleMessageQuote(byte[] buffer)
		{
			try
			{
				int id = BitConverter.ToInt32(buffer, 0);
				string quote = Encoding.ASCII.GetString(buffer, 4, buffer.Length-4);
				Global.battleRoom.PostMessage(id, quote);
				m_ConnectionManager.socketHandler.Read(1, new SocketHandler.delRead(ConnListener), Global.PrimaryForm);
			}
			catch ( Exception ex )
			{
				MessageBox.Show(ex.ToString());
			}
		}

		private void m_Timer_Tick(object sender, EventArgs e)
		{
			try
			{
				lock ( m_ConnectionManager )
				{
					m_ConnectionManager.Send((byte)BATTLEPACK.PACK_KEEPALIVE);
				}
				// if the connection is lost, leave the battle room
				if ( !m_Stopped && !m_ConnectionManager.Connected )
				{
					m_Timer.Stop();
					m_Timer.Dispose();
					m_Timer = null;
					MessageBox.Show("Connection lost from battle host");
					Global.battleRoom.Close();
				}
			}
			catch ( Exception ex )
			{
				MessageBox.Show(ex.ToString());
				Timer timer = (Timer)sender;
				timer.Stop();
				Global.battleRoom.Close();
			}
		}
	}
}
