using System;
using System.Collections;
using System.Windows.Forms;
using System.Threading;
using ClientControls;

namespace SpringClient
{
	/// <summary>
	/// Summary description for ChatroomManager.
	/// </summary>
	public class ChatroomManager
	{
		TabControl		m_ChatTabs;
		ListView		m_ChatroomList;

		SortedList		m_Chatrooms;

		public ChatroomManager(TabControl chatTabs, ListView chatList)
		{
			m_Chatrooms = new SortedList();
			m_ChatTabs = chatTabs;
			m_ChatroomList = chatList;
			//
			// TODO: Add constructor logic here
			//
		}

		public void CloseAllChatrooms()
		{
			foreach ( Chatroom chat in m_Chatrooms.Values )
			{
				TabPage tp = (TabPage)chat.Parent;
				TabControl tc = (TabControl)tp.Parent;
				tc.TabPages.Remove(tp);
			}
			m_Chatrooms.Clear();
		}

		public void AddUserToChatroom(Guid chatid, int userid, string name, string clan, bool notify)
		{
			if ( !m_Chatrooms.ContainsKey(chatid) )
			{
				MessageBox.Show("Could not add user " + userid.ToString() + " to chat number " + chatid.ToString());
				return;
			}

			Chatroom chat = (Chatroom)m_Chatrooms[chatid];
			chat.AddUser(userid, name, clan, notify);
		}

		/*
		public void RemoveChatroom(string val)
		{
			if ( m_ChatroomIDs.Contains(val) )
			{
				m_ChatroomIDs.Remove(val);
				Chatrooms.Items.Remove(val);
			}
		}

		public void RemoveChatroom(int id)
		{
			int index = m_ChatroomIDs.IndexOfValue(id);
			if ( index >= 0 )
				m_ChatroomIDs.RemoveAt(index);
		}

		public int GetID(string val)
		{
			if ( !m_ChatroomIDs.ContainsKey(val) )
				return -1;
			return (int)m_ChatroomIDs[val];
		}

*/
		public void RequestEnterChatroom(ListViewItem lvi)
		{
			Guid id = new Guid(lvi.SubItems[2].Text);
			if ( m_Chatrooms.Contains(id) )
			{
				// dont request to enter a chatroom twice
				return;
			}

			byte[] msg = new byte[17];
			msg[0] = (byte)PACKETS.PACK_REQJOINCHAT;
			Phoenix.ByteConverter.ToNetworkBytes(id,msg,1);
			Global.connMan.Send(msg);
			//Global.connMan.Send(id);
		}

		public void EnterChatroom(Guid id, string name)
		{
			Chatroom cr = new Chatroom();
			cr.Dock = DockStyle.Fill;

			TabPage tp = new TabPage();
			tp.Name = name;
			tp.Text = name;
			tp.Controls.Add(cr);

			m_ChatTabs.Controls.Add(tp);

			m_ChatTabs.SelectedTab = tp;
			cr.ID = id;
			m_Chatrooms.Add(id, cr);
		}

		public void LeaveChatroom(Guid id)
		{
			if ( !m_Chatrooms.ContainsKey(id))
			{
				MessageBox.Show("Could not leave chatroom.  id does not exist");
				return;
			}

			Chatroom chat = (Chatroom)m_Chatrooms[id];
			TabPage tp = (TabPage)chat.Parent;
			TabControl tc = (TabControl)tp.Parent;
			byte[] msg = new byte[1];
			msg[0] = (byte)PACKETS.PACK_LEAVECHAT;

			Global.connMan.Send(msg);
			Global.connMan.Send(id);

			tc.TabPages.Remove(tp);
			m_Chatrooms.Remove(id);
			
		}

		public void ChatMessagePost(int fromid, Guid chatid, string msg)
		{
			if ( !m_Chatrooms.ContainsKey(chatid) )
			{
		//		MessageBox.Show(string.Format("Chat {0} not found.  could not post message", chatid));
				return;
			}

			Chatroom chat = (Chatroom)m_Chatrooms[chatid];
			chat.PostMessage(fromid, msg);
		}

		public void RemoveUserFromChatroom(Guid chatid, int userid)
		{
			if ( !m_Chatrooms.ContainsKey(chatid) )
			{
		//		MessageBox.Show(string.Format("Chat {0} not found.  could not remove user", chatid));
				return;
			}
			
			Chatroom chat = (Chatroom)m_Chatrooms[chatid];
			chat.RemoveUser(userid);
		}
	}
}
