#ifndef __ARCHIVE_SCANNER_H
#define __ARCHIVE_SCANNER_H

#include <string>
#include <vector>
#include <map>

#include <stdio.h>

using namespace std;

/*
 * This class searches through a given directory and its subdirectories looking for archive files.
 * When it finds one, it figures out what kind of archive it is (i.e. if it is a map or a mod currently).
 * This information is cached, so that only modified archives are actually opened. The information can
 * then be retreived by the mod and map selectors.
 *
 * The archive namespace is global, so it is not allowed to have an archive with the same name in more
 * than one folder.
 */

class CSunParser;

class CArchiveScanner
{
public:
	struct MapData {
		string name;
		string virtualPath;					// Where in the archive the map can be found
	};
	struct ModData {
		string name;
		string description;
		int modType;
		vector<string> dependencies;		// Archives it depends on
	};
	CArchiveScanner(void);
	void ReadCacheData();
	void WriteCacheData();
	virtual ~CArchiveScanner(void);
	void Scan(const string& curPath);
	vector<ModData> GetPrimaryMods();
	vector<string> GetArchives(const string& root);
	vector<string> GetMaps();
	vector<string> GetArchivesForMap(const string& mapName);
protected:
	struct ArchiveInfo {
		string path;
		string origName;					// Could be useful to have the non-lowercased name around
		unsigned int modified;
		vector<MapData> mapData;
		ModData modData;
		bool updated;
	};
	map<string, ArchiveInfo> archiveInfo;	
	ModData GetModData(CSunParser* p, const string& section);
	bool isDirty;
	//void WriteModData(FILE* out, const ModData& data);	// Helper to write out dependencies
};

extern CArchiveScanner* archiveScanner;

#endif